import json
import os
from typing import Any, Dict, Union

import requests
from etripy.error import HTTPException
from etripy.model import FileType


class SyncEtriRequest:
    base_url: str = "http://aiopen.etri.re.kr:8000"

    def __init__(self, access_key: str) -> None:
        self.access_key = access_key

    def request(
        self, method: str, endpoint: str, data: Dict[str, Union[str, int]]
    ) -> Dict[str, Any]:
        headers = {
            "Content-Type": "application/json; charset=UTF-8",
            "Authorization": self.access_key,
        }
        url = self.base_url + endpoint
        response = requests.request(method, url=url, headers=headers, json=data)
        rescode = response.status_code
        if rescode == 200:
            return response.json()
        else:
            raise HTTPException(f"Error Code : {rescode} : {response.text}")

    def request_file_upload(self, data: Dict[str, Any]) -> Dict[str, Any]:
        headers = {"Authorization": self.access_key}
        url = self.base_url + "/DocUpload"
        response = requests.request(
            "POST",
            url=url,
            headers=headers,
            data={"json": data["json"]},
            files={"doc_file": data["doc_file"]},
        )
        rescode = response.status_code
        if rescode == 200:
            return response.json()
        else:
            raise HTTPException(f"Error Code : {rescode} : {response.text}")

    def get_analysis_data(self, data: Dict[str, Union[str, int]], spoken: bool):
        if spoken:
            return self.request(method="POST", endpoint="/WiseNLU_spoken", data=data)
        else:
            return self.request(method="POST", endpoint="/WiseNLU", data=data)

    def file_upload(
        self, upload_file_path: str, file_type: Union[FileType, str] = "hwp"
    ):
        with open(upload_file_path, "rb") as file:
            file_content = file.read()

        requestJson = {"argument": {"type": file_type}}

        data = {
            "json": json.dumps(requestJson),
            "doc_file": (os.path.basename(upload_file_path), file_content),
        }
        return self.request_file_upload(data=data)



class HTTPException(Exception):
    pass


class SentencesException(Exception):
    pass


class QAException(Exception):
    pass


class AnalysisException(Exception):
    pass


class AudioFileException(Exception):
    pass


class VoiceRecognitionException(Exception):
    pass


class VoicePronunciationException(Exception):
    pass



import pytest
from etripy import sync

access_key = ""  # 엑세스 키


@pytest.fixture
def analysis():
    return sync.AnalysisClient(access_key=access_key)


@pytest.fixture
def qa():
    return sync.QAClient(access_key=access_key)



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from .analysis import AnalysisResult
from .base import BaseQAEtri


@dataclass(frozen=True)
class VQT:
    """의문사기반 질문유형"""

    qt: Optional[int] = field(repr=True, compare=True, default=None)
    """질문유형 코드(숫자)"""
    strQTClue: Optional[int] = field(repr=True, compare=True, default=None)
    """의문사 스트링"""


@dataclass(frozen=True)
class VQF:
    """질문초점 객체 배열"""

    strQF: Optional[int] = field(repr=True, compare=True, default=None)
    """질문초점 스트링"""
    dWeightQF: Optional[int] = field(repr=True, compare=True, default=None)
    """질문초점 신뢰도"""


@dataclass(frozen=True)
class VLAT:
    """어휘정답유형 객체 배열"""

    strLAT: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘정답유형"""
    vCompoundLATs: Optional[List[int]] = field(repr=True, compare=True, default=None)
    """어휘정답유형의 복합명사 형태 스트링 목록"""
    strID: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 코드(의미 또는 개념망 ID)"""
    dConfidenceLAT: Optional[float] = field(repr=True, compare=True, default=None)
    """인식의 신뢰도"""


@dataclass(frozen=True)
class VSAT:
    """세분류 의미정답유형 객체 배열"""

    strSAT: Optional[str] = field(repr=True, compare=True, default=None)
    """의미정답유형 스트링"""
    expanse_SAT: Optional[List[int]] = field(repr=True, compare=True, default=None)
    """확장 의미정답유형 배열"""
    dConfidenceSAT: Optional[float] = field(repr=True, compare=True, default=None)
    """인식 신뢰도"""


@dataclass(frozen=True)
class VEntityInfo:
    """개체의 위키백과 정보"""

    strNormEntity: Optional[str] = field(repr=True, compare=True, default=None)
    """위키백과 타이틀로 정규화된 개체 스트링"""
    strExplain: Optional[str] = field(repr=True, compare=True, default=None)
    """타이틀의 정의문"""
    dWeightEn: Optional[float] = field(repr=True, compare=True, default=None)
    """모호성 해소 가중치"""


@dataclass(frozen=True)
class VSATRoot:
    """대분류 의미정답유형 객체 배열"""

    strSAT: Optional[str] = field(repr=True, compare=True, default=None)
    """의미정답유형 스트링"""
    expanse_SAT: Optional[List[int]] = field(repr=True, compare=True, default=None)
    """확장 의미정답유형 배열"""
    dConfidenceSAT: Optional[float] = field(repr=True, compare=True, default=None)
    """인식 신뢰도"""


@dataclass(frozen=True)
class VTitleTopic:
    """주요한 개체 정보 배열"""

    strEntity: Optional[str] = field(repr=True, compare=True, default=None)
    """개체 스트링"""
    strEntityType: Optional[str] = field(repr=True, compare=True, default=None)
    """개체의 NE 스트링"""
    dWeightTitle: Optional[float] = field(repr=True, compare=True, default=None)
    """개체 중요도"""
    vEntityInfo: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    """개체의 위키백과 정보"""

    @property
    def VEntityInfo(self) -> List[VEntityInfo]:
        """개체의 위키백과 정보"""
        if self.vEntityInfo is None:
            return []
        return [VEntityInfo(**info) for info in self.vEntityInfo]


@dataclass(frozen=True)
class OrgQUnit:
    """질문분석 기본정보"""

    strQuestion: Optional[str] = field(repr=True, compare=True, default=None)
    """질문 스트링"""
    strTaggedQ: Optional[str] = field(repr=True, compare=True, default=None)
    """질문 태깅결과 스트링(lexico-semantic pattern 매칭을 위한 언어분석 결과 출력"""
    ndoc: Optional[Dict[str, Any]] = field(repr=True, compare=True, default=None)
    """질문의 언어분석 결과 객체"""
    vQTs: Optional[List[Dict[str, Any]]] = field(repr=True, compare=True, default=None)
    """의문사기반 질문유형"""
    vQFs: Optional[List[Dict[str, Any]]] = field(repr=True, compare=True, default=None)
    """질문초점 객체 배열"""
    vLATs: Optional[List[Dict[str, Any]]] = field(repr=True, compare=True, default=None)
    """어휘정답유형 객체 배열"""
    vSATs: Optional[List[Dict[str, Any]]] = field(repr=True, compare=True, default=None)
    """세분류 의미정답유형 객체 배열"""
    vSATRoots: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    """대분류 의미정답유형 객체 배열"""
    vTitles: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    """주요한 개체 정보 배열"""
    vQTopic: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    """위키백과 타이틀 중 가장 중요한 타이틀"""
    answerConstraint: Optional[List[Any]] = field(repr=True, compare=True, default=None)
    dIntegrativeConf: Optional[float] = field(repr=True, compare=True, default=None)
    """LAT와 SAT의 통합 신뢰도 정보"""

    @property
    def Ndoc(self) -> AnalysisResult:
        """질문의 언어분석 결과 객체"""
        if self.ndoc is None:
            return AnalysisResult(data={})
        return AnalysisResult(data=self.ndoc, **self.ndoc)

    @property
    def VQTs(self) -> List[VQT]:
        """의문사기반 질문유형"""
        if self.vQTs is None:
            return []
        return [VQT(**vqt) for vqt in self.vQTs]

    @property
    def VQFs(self) -> List[VQF]:
        """질문초점 객체 배열"""
        if self.vQFs is None:
            return []
        return [VQF(**vqf) for vqf in self.vQFs]

    @property
    def VLATs(self) -> List[VLAT]:
        """어휘정답유형 객체 배열"""
        if self.vLATs is None:
            return []
        return [VLAT(**vlat) for vlat in self.vLATs]

    @property
    def VSATs(self) -> List[VSAT]:
        """어휘정답유형 객체 배열"""
        if self.vSATs is None:
            return []
        return [VSAT(**vsat) for vsat in self.vSATs]

    @property
    def VSATRoots(self) -> List[VSATRoot]:
        """대분류 의미정답유형 객체 배열"""
        if self.vSATRoots is None:
            return []
        return [VSATRoot(**vsatr) for vsatr in self.vSATRoots]

    @property
    def VTitles(self) -> List[VTitleTopic]:
        """주요한 개체 정보 배열"""
        if self.vTitles is None:
            return []
        return [VTitleTopic(**vtitle) for vtitle in self.vTitles]

    @property
    def VQTopic(self) -> List[VTitleTopic]:
        """위키백과 타이틀 중 가장 중요한 타이틀"""
        if self.vQTopic is None:
            return []
        return [VTitleTopic(**vqtopic) for vqtopic in self.vQTopic]


@dataclass(frozen=True)
class OrgQInfo:
    """질문분석 기본정보"""

    orgQUnit: Optional[Dict[str, Any]] = field(
        repr=True, compare=True, default_factory=dict
    )  # default_factory를 사용하여 기본값 설정
    """원질문 정보 객체"""

    @property
    def OrgQUnit(self) -> OrgQUnit:
        """원질문 정보 객체"""
        if not self.orgQUnit:
            return OrgQUnit()
        return OrgQUnit(**self.orgQUnit)


@dataclass(frozen=True)
class AnsQType:
    """정답형태에 따른 질문유형 객체"""

    strQType4Chg: Optional[str] = field(repr=True, compare=True, default=None)
    """질문유형 스트링"""
    dWeightCQT: Optional[float] = field(
        repr=True, compare=True, default=None
    )  # 기본값을 None에서 float로 변경
    """인식 신뢰도"""


@dataclass(frozen=True)
class VSemQType:
    """의미적 질문유형 객체"""

    strQType4Chg: Optional[str] = field(repr=True, compare=True, default=None)
    """질문유형 스트링"""
    dWeightCQT: Optional[float] = field(
        repr=True, compare=True, default=None
    )  # 기본값을 None에서 float로 변경
    """인식 신뢰도"""


@dataclass(frozen=True)
class QClassification:
    """질문분류 정보 객체"""

    ansQType: Optional[Dict[str, Any]] = field(
        repr=True, compare=True, default_factory=dict
    )  # default_factory를 사용하여 기본값 설정
    """정답형태에 따른 질문유형 객체"""
    vSemQType: Optional[Dict[str, Any]] = field(
        repr=True, compare=True, default_factory=dict
    )  # default_factory를 사용하여 기본값 설정
    """의미적 질문유형 객체"""

    @property
    def AnsQType(self) -> AnsQType:
        """정답형태에 따른 질문유형 객체"""
        if not self.ansQType:
            return AnsQType()
        return AnsQType(**self.ansQType)

    @property
    def VSemQType(self) -> VSemQType:
        """의미적 질문유형 객체"""
        if not self.vSemQType:
            return VSemQType()
        return VSemQType(**self.vSemQType)


@dataclass(frozen=True)
class WiseQAnalResult(BaseQAEtri):
    """질의응답 결과 객체"""

    @property
    def OrgQInfo(self) -> OrgQInfo:
        """질문분석 기본정보"""
        return OrgQInfo(**self.data.get("return_object", {}).get("orgQInfo", {}))

    @property
    def QClassification(self) -> QClassification:
        """질문분류 정보 객체"""
        return QClassification(
            **self.data.get("return_object", {}).get("QClassification", {})
        )



import pytest
from etripy import client

access_key = ""  # 엑세스 키


@pytest.fixture
def analysis():
    return client.AnalysisClient(access_key=access_key)


@pytest.fixture
def qa():
    return client.QAClient(access_key=access_key)



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from .base import BaseLanguageEtri


####################### 패러플라이즈 #######################
@dataclass(frozen=True)
class ParaphraseResult(BaseLanguageEtri):
    result: Optional[str] = field(repr=True, compare=True, default=None)
    """패러프레이즈 분석 결과"""

    @property
    def is_paraphrase(self) -> bool:
        """패러프레이즈 분석 결과"""
        return self.result == "paraphrase"


####################### 어휘 정보 #######################
@dataclass(frozen=True)
class MetaInfo:
    Title: Optional[str] = field(repr=True, compare=True, default=None)
    """Open APIs의 타이틀 명"""
    Link: Optional[str] = field(repr=True, compare=True, default=None)
    """해당 Open APIs를 사용하기 위한 URL 정보"""


@dataclass(frozen=True)
class WordInfoList:
    """어휘의 상세 정보"""

    PolysemyCode: Optional[int] = field(repr=True, compare=True, default=None)
    """어휘의 다의어 코드 (2자리 숫자)"""
    Definition: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 의미 정보"""
    POS: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 품사"""
    Hypernym: Optional[List[str]] = field(repr=True, compare=True, default=None)
    """어휘의 상위 어휘 정보"""
    Hypornym: Optional[List[str]] = field(repr=True, compare=True, default=None)
    """어휘의 하위 어휘 정보"""


@dataclass(frozen=True)
class WordInfo:
    """어휘의 상세 정보"""

    Word: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 원문자열"""
    HomonymCode: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 동음이의어 코드 (2자리 숫자)"""
    WordInfo: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    """어휘의 상세 정보"""
    Synonym: Optional[List[str]] = field(repr=True, compare=True, default=None)
    """어휘의 유의어 어휘 정보"""
    Antonym: Optional[List[str]] = field(repr=True, compare=True, default=None)
    """어휘의 반의어 어휘 정보"""

    @property
    def WordInfoList(self) -> List[WordInfoList]:
        """어휘의 상세 정보"""
        if self.WordInfo is None:
            return []
        return [WordInfoList(**wi) for wi in self.WordInfo]


@dataclass(frozen=True)
class WordResult(BaseLanguageEtri):
    """어휘 정보 결과"""

    @property
    def MetaInfo(self) -> MetaInfo:
        """Open APIs의 정보"""
        return MetaInfo(**self.data["return_object"]["MetaInfo"])

    @property
    def WordInfo(self) -> List[WordInfo]:
        """어휘의 상세 정보"""
        return [
            WordInfo(**wordinfo)
            for wordinfo in self.data["return_object"]["WWN WordInfo"]
        ]


####################### 동음이의어 #######################
@dataclass(frozen=True)
class Homonym:
    word: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 원문자열"""
    homonym_code: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 동음이의어 코드 (표준국어대사전 어깨번호)"""
    polysemy_code: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 다의어 코드 (표준국어대사전 어깨번호)"""
    word_class: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 품사"""
    description: Optional[str] = field(repr=True, compare=True, default=None)
    """의미 정보"""


@dataclass(frozen=True)
class HomonymResult(BaseLanguageEtri):
    """동음이의어 정보 결과"""

    homonym: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    """동음이의어 정보"""

    @property
    def Homonym(self) -> List[Homonym]:
        """동음이의어 정보"""
        if self.homonym is None:
            return []
        return [Homonym(**homonym) for homonym in self.homonym]


####################### 다의어 #######################
@dataclass(frozen=True)
class Polysemy:
    word: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 원문자열"""
    homonym_code: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 동음이의어 코드 (표준국어대사전 어깨번호)"""
    polysemy_code: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 다의어 코드 (표준국어대사전 어깨번호)"""
    word_class: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 품사"""
    description: Optional[str] = field(repr=True, compare=True, default=None)
    """의미 정보"""


@dataclass(frozen=True)
class PolysemyResult(BaseLanguageEtri):
    """다의어 정보 결과"""

    polysemy: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    """다의어 정보"""

    @property
    def Polysemy(self) -> List[Polysemy]:
        """다의어 정보"""
        if self.polysemy is None:
            return []
        return [Polysemy(**polysemy) for polysemy in self.polysemy]


####################### 어휘 간 유사도 분석 #######################
@dataclass(frozen=True)
class SimilarityList:
    Algorithm: Optional[str] = field(repr=True, compare=True, default=None)
    """거리 유사도 알고리즘"""
    SimScore: Optional[float] = field(repr=True, compare=True, default=None)
    """거리 유사도 신뢰도"""


@dataclass(frozen=True)
class WordRelInfo_:
    ShortedPath: Optional[List[str]] = field(repr=True, compare=True, default=None)
    """가장 가까운 연결 어휘 경로 정보"""
    Distance: Optional[int] = field(repr=True, compare=True, default=None)
    """어휘 경로 거리 정보 (숫자)"""
    Similarity: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    """어휘 간 거리 유사도"""

    @property
    def SimilarityList(self) -> List[SimilarityList]:
        """어휘 간 거리 유사도"""
        if self.Similarity is None:
            return []
        return [SimilarityList(**sim) for sim in self.Similarity]


@dataclass(frozen=True)
class FirstWordInfo_:
    """첫 번째 어휘 정보"""

    Word: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 원문자열"""
    HomonymCode: Optional[int] = field(repr=True, compare=True, default=None)
    """어휘의 동음이의어 코드 (2자리 숫자)"""
    PolysemyCode: Optional[int] = field(repr=True, compare=True, default=None)
    """어휘의 다의어 코드 (2자리 숫자)"""
    Definition: Optional[str] = field(repr=True, compare=True, default=None)
    """의미 정보"""
    POS: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 품사"""


@dataclass(frozen=True)
class SecondWordInfo_:
    """두 번째 어휘 정보"""

    Word: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 원문자열"""
    HomonymCode: Optional[int] = field(repr=True, compare=True, default=None)
    """어휘의 동음이의어 코드 (2자리 숫자)"""
    PolysemyCode: Optional[int] = field(repr=True, compare=True, default=None)
    """어휘의 다의어 코드 (2자리 숫자)"""
    Definition: Optional[str] = field(repr=True, compare=True, default=None)
    """의미 정보"""
    POS: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 품사"""


@dataclass(frozen=True)
class WordRelInfo:
    """어휘 간 유사도 분석 정보"""

    FirstWordInfo: Optional[Dict[str, Any]] = field(
        repr=True, compare=True, default=None
    )
    """첫 번째 어휘 정보"""
    SecondWordInfo: Optional[Dict[str, Any]] = field(
        repr=True, compare=True, default=None
    )
    """두 번째 어휘 정보"""
    WordRelInfo: Optional[Dict[str, Any]] = field(repr=True, compare=True, default=None)
    """어휘 간 유사도 분석 정보"""

    @property
    def FirstWordInfo_(self) -> FirstWordInfo_:
        """첫 번째 어휘 정보"""
        if self.FirstWordInfo is None:
            return FirstWordInfo_()
        return FirstWordInfo_(**self.FirstWordInfo)

    @property
    def SecondWordInfo_(self) -> SecondWordInfo_:
        """두 번째 어휘 정보"""
        if self.SecondWordInfo is None:
            return SecondWordInfo_()
        return SecondWordInfo_(**self.SecondWordInfo)

    @property
    def WordRelInfo_(self) -> WordRelInfo_:
        """어휘 간 유사도 분석 정보"""
        if self.WordRelInfo is None:
            return WordRelInfo_()
        return WordRelInfo_(**self.WordRelInfo)


@dataclass(frozen=True)
class WordRelResult(BaseLanguageEtri):
    """어휘 간 유사도 분석 결과"""

    @property
    def MetaInfo(self) -> MetaInfo:
        """Open APIs의 정보"""
        return MetaInfo(**self.data["return_object"]["MetaInfo"])

    @property
    def WordRelInfo(self) -> WordRelInfo:
        """어휘 간 유사도 분석 정보"""
        return WordRelInfo(**self.data["return_object"]["WWN WordRelInfo"])


####################### 개체 연결 #######################
@dataclass(frozen=True)
class NELinkingMention:
    mention: Optional[str] = field(repr=True, compare=True, default=None)
    """개체명 문자열"""
    link: Optional[str] = field(repr=True, compare=True, default=None)
    """개체명 위키피디아 링크"""


@dataclass(frozen=True)
class NELinkingResult(BaseLanguageEtri):
    mentions: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    """개체명 연결 정보"""
    sent_id: Optional[int] = field(repr=True, compare=True, default=None)
    """문장 번호"""
    sentence: Optional[str] = field(repr=True, compare=True, default=None)
    """문장"""
    t_sentence: Optional[str] = field(repr=True, compare=True, default=None)
    """번역 문장"""

    @property
    def Mentions(self) -> List[NELinkingMention]:
        """개체명 연결 정보"""
        if self.mentions is None:
            return []
        return [NELinkingMention(**mention) for mention in self.mentions]


####################### 코어퍼런스 #######################
@dataclass(frozen=True)
class CoreferenceEntity:
    text: Optional[str] = field(repr=True, compare=True, default=None)
    """개체명 문자열"""
    type: Optional[str] = field(repr=True, compare=True, default=None)
    """개체명 유형"""
    entity: Optional[List[str]] = field(repr=True, compare=True, default=None)
    """개체명 연결 문자열"""
    animacy: Optional[str] = field(repr=True, compare=True, default=None)
    """개체명 유무생물 정보"""
    gender: Optional[str] = field(repr=True, compare=True, default=None)
    """개체명 성별"""
    number: Optional[str] = field(repr=True, compare=True, default=None)
    """개체명 수"""


@dataclass(frozen=True)
class CoreferenceResult(BaseLanguageEtri):
    entity: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    """코어퍼런스 분석 결과"""

    @property
    def Entity(self) -> List[CoreferenceEntity]:
        """코어퍼런스 분석 결과"""
        if self.entity is None:
            return []
        entity_ = []
        for entity in self.entity:
            if "animacy " in entity:
                entity["animacy"] = entity["animacy "]
                del entity["animacy "]
            entity_.append(entity)
        return [CoreferenceEntity(**entity) for entity in entity_]



import json
import os
from typing import Any, Dict, Union

import aiohttp
from etripy.error import HTTPException
from etripy.model import FileType


class EtriRequest:
    base_url: str = "http://aiopen.etri.re.kr:8000"

    def __init__(self, access_key: str) -> None:
        self.access_key = access_key

    async def request(
        self, method: str, endpoint: str, data: Dict[str, Any]
    ) -> Dict[str, Any]:
        headers = {
            "Content-Type": "application/json; charset=UTF-8",
            "Authorization": self.access_key,
        }

        url = self.base_url + endpoint
        async with aiohttp.ClientSession(headers=headers) as session:
            async with session.request(method, url=url, json=data) as response:
                rescode = response.status
                if rescode == 200:
                    return await response.json()
                else:
                    text_data = await response.text()
                    raise HTTPException(f"Error Code {rescode} : {text_data}")

    async def request_file_upload(self, data: aiohttp.FormData) -> Dict[str, Any]:
        headers = {
            "Authorization": self.access_key,
        }

        url = self.base_url + "/DocUpload"
        async with aiohttp.ClientSession(headers=headers) as session:
            async with session.request("POST", url=url, data=data) as response:
                rescode = response.status
                if rescode == 200:
                    return await response.json()
                else:
                    text_data = await response.text()
                    raise HTTPException(f"Error Code {rescode} : {text_data}")

    async def get_analysis_data(self, data: Dict[str, Any], spoken: bool):
        if spoken:
            return await self.request(
                method="POST", endpoint="/WiseNLU_spoken", data=data
            )
        else:
            return await self.request(method="POST", endpoint="/WiseNLU", data=data)

    async def file_upload(
        self, upload_file_path: str, file_type: Union[FileType, str] = "hwp"
    ):
        file = open(upload_file_path, "rb")
        file_content = file.read()
        file.close()

        requestJson = {"argument": {"type": file_type}}

        form_data = aiohttp.FormData()
        form_data.add_field(
            "json", json.dumps(requestJson), content_type="application/json"
        )
        form_data.add_field(
            "doc_file",
            file_content,
            filename=os.path.basename(file.name),
            content_type="application/octet-stream",
        )
        return await self.request_file_upload(data=form_data)



from typing import Any, Collection, Dict, List, Optional, Tuple, Union

from etripy.error import AnalysisException, QAException, SentencesException
from etripy.model import AnalysisCode, FileType, WikiType
from etripy.model.language import (
    AnalysisResult,
    CoreferenceResult,
    DocResult,
    DocUploadResult,
    HomonymResult,
    LegalResult,
    MRCResult,
    NELinkingResult,
    ParaphraseResult,
    PolysemyResult,
    WiKiResult,
    WiseQAnalResult,
    WordRelResult,
    WordResult,
)
from etripy.sync.http import SyncEtriRequest


class AnalysisClient(SyncEtriRequest):
    """
    ETRI 언어 처리 및 분석 클라이언트 클래스입니다. (동기 처리)
    #### access_key
    [ETRI 포털사이트](https://aiopen.etri.re.kr/)에서 발급받은 `access_key`를 입력합니다.
    """

    def __init__(self, access_key: str) -> None:
        super().__init__(access_key=access_key)

    def analysis(
        self, text: str, analysis_code: Union[AnalysisCode, str], spoken: bool = False
    ) -> Optional[AnalysisResult]:
        """
        ### - 언어 분석
        문어체 언어분석 및 구어체 언어분석 결과를 제공합니다.

        #### Parameter
        `analysis_code` : 요청할 분석 코드 (AnalysisCode 클래스 사용 권장)\n
        `text` : 분석할 자연어 문장으로서 UTF-8 인코딩된 텍스트만 지원
        """
        data: Dict[str, Union[str, int]] = {
            "argument": {
                "analysis_code": str(analysis_code),  # 문자열로 변환
                "text": text,
            }
        }
        result = self.get_analysis_data(data=data, spoken=spoken)
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise AnalysisException(result["reason"])
        return AnalysisResult(data=result, **result["return_object"])

    def paraphrase(self, *sentences: str) -> Optional[ParaphraseResult]:
        """
        ### - 문장 패러프레이즈 인식
        문장 패러프레이즈 인식 API는 두 개의 문장이 동등한 의미를 가지는지 여부를 판별합니다.

        #### Parameter
        `sentences` : 분석할려는 문장에 대한 텍스트 (두 문장만 입력해주세요.)
        """
        if len(sentences) != 2:
            raise SentencesException(f"두 문장만 입력해주세요.\n현재 입력한 문장 수 : {len(sentences)}")
        data = {
            "argument": {"sentence1": sentences[0], "sentence2": sentences[1]},
        }
        result = self.request(method="POST", endpoint="/ParaphraseQA", data=data)
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise AnalysisException(result["reason"])
        return ParaphraseResult(data=result, **result["return_object"])

    def wordinfo(self, word: str) -> Optional[WordResult]:
        """
        ### - 어휘 정보
        다양한 어휘지식을 통합한 WiseWordNet 어휘 지식베이스에 기반하여 어휘의 정보를 분석하는 기술로서 입력된 어휘에 대한 관련 제공합니다.

        #### Parameter
        `word` : 분석할 어휘 Text 로서 UTF-8 인코딩된 텍스트만 지원
        """
        data: Dict[str, Union[str, int]] = {"argument": {"word": word}}
        result = self.request(method="POST", endpoint="/WiseWWN/Word", data=data)
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise AnalysisException(result["reason"])
        return WordResult(data=result)

    def homonym(self, word: str) -> Optional[HomonymResult]:
        """
        ### - 동음이의어 정보
        국립국어원의 표준국어대사전에 등재된 어휘의 동음이의어(소리는 같으나 뜻이 다른 단어) 사전 정보를 조회하는 API로 입력된 어휘의 동음이의어 정보를 제공합니다.

        #### Parameter
        `word` : 동음이의어를 조회할 어휘 Text 로서 UTF-8 인코딩된 텍스트만 지원
        """
        data: Dict[str, Union[str, int]] = {"argument": {"word": word}}
        result = self.request(method="POST", endpoint="/WiseWWN/Homonym", data=data)
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise AnalysisException(result["reason"])
        return HomonymResult(data=result, **result["return_object"])

    def polysemy(
        self, word: str, homonym_code: Optional[str] = None
    ) -> Optional[PolysemyResult]:
        """
        ### - 다의어 정보
        국립국어원의 표준국어대사전에 등재된 어휘의 다의어(두 가지 이상의 뜻을 가진 단어) 사전 정보를 조회하는 API로 입력된 어휘의 다의어 정보를 제공합니다.

        #### Parameter
        `word` : 다의어를 조회할 어휘 Text 로서 UTF-8 인코딩된 텍스트만 지원\n
        `homonym_code` : 다의어를 조회할 어휘의 동음이의어 코드 (필수 X)
        """
        data: Dict[str, Union[str, int]] = {"argument": {"word": word}}
        if homonym_code:
            data["homonym_code"] = homonym_code
        result = self.request(method="POST", endpoint="/WiseWWN/Polysemy", data=data)
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise AnalysisException(result["reason"])
        return PolysemyResult(data=result, **result["return_object"])

    def wordrel(
        self,
        first_word: str,
        second_word: str,
        first_sense_id: Optional[str] = None,
        second_sense_id: Optional[str] = None,
    ) -> Optional[WordRelResult]:
        """
        ### - 어휘 간 유사도 분석
        다양한 어휘지식을 통합한 WiseWordNet 어휘 지식베이스에 기반하여 어휘 간 거리 정보를 분석하는 기술로서 입력된 여휘간 유사도 결과를 제공합니다.

        #### Parameter
        `first_word` : 비교 분석할 어휘 Text 로서 UTF-8 인코딩된 텍스트만 지원\n
        `first_sense_id` : 첫 번째 어휘의 의미 코드 (필수 X)\n
        `second_word` : 비교 분석 대상 어휘 Text 로서 UTF-8 인코딩된 텍스트만 지원\n
        `second_sense_id` : 두 번째 어휘의 의미 코드 (필수 X)\n
        """
        data: Dict[str, Union[str, int]] = {
            "argument": {"first_word": first_word, "second_word": second_word}
        }
        if first_sense_id:
            data["first_sense_id"] = first_sense_id
        if second_sense_id:
            data["second_sense_id"] = second_sense_id
        result = self.request(method="POST", endpoint="/WiseWWN/WordRel", data=data)
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise AnalysisException(result["reason"])
        return WordRelResult(data=result)

    def nelinking(self, contents: str) -> Optional[List[NELinkingResult]]:
        """
        ### - 개체 연결(NE Linking)
        개체 연결(entity linking) API는 문장 내에서 인식된 개체 멘션(entity mention)을 지식베이스의 개체(entity)와 연결하는 기술을 제공합니다.

        #### Parameter
        `contents` : 분석할 문단
        """
        data = {"argument": {"contents": contents}}
        result = self.request(method="POST", endpoint="/NELinking", data=data)
        print(result)
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise AnalysisException(result["reason"])
        return [
            NELinkingResult(data=result, **result_)
            for result_ in result["return_object"]
        ]

    def coreference(self, text: str) -> Optional[CoreferenceResult]:
        """
        ### - 상호참조 해결
        상호참조 해결(coreference resolution) API는 어떤 개체에 대한 여러 표현들이 이루고 있는 참조관계를 밝히는 기술을 제공합니다.

        #### Parameter
        `text` : 분석할 문단
        """
        data: Dict[str, Union[str, int]] = {"argument": {"text": text}}
        result = self.request(method="POST", endpoint="/Coreference", data=data)
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise AnalysisException(result["reason"])
        result_ = {"entity": result["return_object"]["entity"]}
        return CoreferenceResult(data=result, **result_)


class QAClient(SyncEtriRequest):
    """
    ETRI 질의응답 클라이언트 클래스입니다. (동기 처리)
    #### access_key
    [ETRI 포털사이트](https://aiopen.etri.re.kr/)에서 발급받은 `access_key`를 입력합니다.
    """

    def __init__(self, access_key: str) -> None:
        super().__init__(access_key=access_key)

    def qanal(self, text: str) -> Optional[WiseQAnalResult]:
        """
        ### - 질문분석
        자연어 질문을 분석하여 의미를 이해하고 구조화하는 기술을 제공합니다.

        #### Parameter
        `text` : 분석할 질문 Text 로서 UTF-8 인코딩된 텍스트만 지원합니다.
        """
        data: Dict[str, Union[str, int]] = {
            "text": text,
        }
        result = self.request(method="POST", endpoint="/WiseQAnal", data=data)
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise QAException(result["reason"])
        return WiseQAnalResult(data=result)

    def mrcservlet(self, question: str, passage: str) -> Optional[MRCResult]:
        """
        ### - 기계독해
        자연어로 쓰여진 단락과 사용자 질문이 주어졌을 때, 딥러닝 기술을 이용하여 단락 중 정답에 해당하는 영역을 찾는 기술을 제공합니다.

        #### Parameter
        `question` : 질문하고자 하는 Text 로서 UTF-8 인코딩된 텍스트만 지원\n
        `passage` : 질문의 답이 포함된 Text 로서 UTF-8 인코딩된 텍스트만 지원
        """
        data: Dict[str, Union[str, int]] = {
            "passage": passage,
            "question": question,
        }
        result = self.request(method="POST", endpoint="/MRCServlet", data=data)
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise QAException(result["reason"])
        return MRCResult(data=result["return_object"]["MRCInfo"])

    def wiki(self, type: Union[WikiType, str], question: str) -> Optional[WiKiResult]:
        """
        ### - 위키백과 QA
        자연어로 기술된 질문의 의미를 분석하여, 위키백과 문서에서 정답과 신뢰도 및 검색 단락을 추론하여 제공합니다.

        #### Parameter
        `question` : 질문하고자 하는 Text 로서 UTF-8 인코딩된 텍스트만 지원\n
        `type` : 질문 응답 엔진의 종류 로서 UTF-8 인코딩된 텍스트만 지원 (WikiType 클래스 사용 추천)
        """
        data: Dict[str, Union[str, int]] = {
            "type": str(type),  # 문자열로 변환
            "question": question,
        }
        result = self.request(method="POST", endpoint="/WikiQA", data=data)
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise QAException(result["reason"])
        return WiKiResult(data=result)

    def legal(self, question: str) -> Optional[LegalResult]:
        """
        ### - 법률 QA
        자연어로 기술된 질문의 의미를 분석하여, 법령문서에서 조 내용을 검색하고 정답을 추론하여 제공합니다.

        #### Parameter
        `question` : 질문하고자 하는 Text 로서 UTF-8 인코딩된 텍스트만 지원\n
        """
        data: Dict[str, Union[str, int]] = {
            "question": question,
        }
        result = self.request(method="POST", endpoint="/LegalQA", data=data)
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise QAException(result["reason"])
        return LegalResult(data=result)

    # API 측에서 서비스 종료로 인한 코드 주석 처리
    # def doc_upload(
    #     self,
    #     upload_file_path: str,
    #     file_type: Union[FileType, str] = "hwp",
    #     message: bool = False,
    # ) -> Optional[DocUploadResult]:
    #     """
    #     ### - 행정문서 QA (문서등록)
    #     행정문서로 작성된 행정문서의 내용을 이해하여 사용자의 자연어 질문에 올바른 답과 근거를 제공합니다.

    #     #### Parameter
    #     `upload_file_path` : 업로드 할려는 문서 파일의 경로\n
    #     `file_type` : 업로드 할려는 문서 파일의 확장자(hwp/hwpx)\n
    #     `message` : 파일 업로드 메세지를 출력합니다.
    #     """
    #     if message:
    #         print("파일 업로드 중...")
    #     result = self.file_upload(
    #         upload_file_path=upload_file_path, file_type=file_type
    #     )
    #     try:
    #         if result["return_object"] == {}:
    #             return None
    #     except KeyError:
    #         if result["result"] != "0":
    #             raise QAException(result["reason"])
    #     return DocUploadResult(data=result)

    # def doc(self, doc_key: str, question: str) -> Optional[DocResult]:
    #     """
    #     ### - 행정문서 QA (질의응답)
    #     행정문서로 작성된 행정문서의 내용을 이해하여 사용자의 자연어 질문에 올바른 답과 근거를 제공합니다.

    #     #### Parameter
    #     `doc_key` : 문서 등록 API에서 리턴 받은 doc key 로서 UTF-8 인코딩된 텍스트만 지원\n
    #     `question` : 질문하고자 하는 text로서 UTF-8 인코딩된 텍스트만 지원
    #     """
    #     data: Dict[str, Union[str, int]] = {
    #         "doc_key": doc_key,
    #         "question": question,
    #     }
    #     result = self.request(method="POST", endpoint="/DocQA", data=data)
    #     try:
    #         if result["return_object"] == {}:
    #             return None
    #     except KeyError:
    #         if result["result"] != "0":
    #             raise QAException(result["reason"])
    #     return DocResult(data=result)



class AnalysisCode:
    """분석 타입을 반환하는 클래스입니다."""

    morp: str = "morp"
    """형태소 분석 (문어/구어)"""

    wsd: str = "wsd"
    """어휘의미 분석 (동음이의어 분석)(문어)"""

    wsd_poly: str = "wsd_poly"
    """어휘의미 분석 (다의어 분석)(문어)"""

    ner: str = "ner"
    """개체명 인식 (문어/구어)"""

    dparse: str = "dparse"
    """의존 구문 분석 (문어)"""

    srl: str = "srl"
    "의미역 인식 (문어)"


class WikiType:
    """질문 응답 엔진의 종류를 반환하는 클래스입니다."""

    irqa: str = "irqa"
    """언어분석 기반과 기계독해 기반의 질의응답을 통합한 질의응답 방식"""

    kbqa: str = "kbpa"
    """지식베이스 기반의 질의응답 방식"""

    hybridqa: str = "hybridqa"
    """"irqa와 kbqa를 통합한 질의응답 방식"""


class FileType:
    """행정문서 QA에서 한글 문서 확장자를 선언할때 사용하는 클래스입니다."""

    hwp: str = "hwp"
    """hwp 문서를 분석할 때 사용합니다."""

    hwpx: str = "hwpx"
    """hwpx 문서를 분석할 때 사용합니다."""


class LanguageCodeType:
    """음성인식의 입력 음성 언어 코드입니다."""

    korean: str = "korean"
    """한국어 음성인식 코드"""

    english: str = "english"
    """영어 음성인식 코드"""

    japanese: str = "japanese"
    """일본어 음성인식 코드"""

    chinese: str = "chinese"
    """중국어 음성인식 코드"""

    spanish: str = "spanish"
    """스페인어 음성인식 코드"""

    french: str = "french"
    """프랑스어 음성인식 코드"""

    german: str = "german"
    """독일어 음성인식 코드"""

    russian: str = "russian"
    """러시아어 음성인식 코드"""

    vietnam: str = "vietnam"
    """베트남어 음성인식 코드"""

    arabic: str = "arabic"
    """아랍어 음성인식 코드"""

    thailand: str = "thailand"
    """태국어 음성인식 코드"""

    portuguese: str = "portuguese"
    """포르투칼어 음성인식 코드"""

    malaysia: str = "malaysia"
    """말레이어 음성인식 코드"""

    indonesian: str = "indonesian"
    """인도네시아어 음성인식 코드"""

    cantonese: str = "cantonese"
    """광둥어 음성인식 코드"""



import pytest
from etripy.client import AnalysisClient, AnalysisCode

analysis_text = "엑소브레인은 내 몸 바깥에 있는 인공 두뇌라는 뜻으로, 세계 최고인공지능 기술 선도라는 비전을 달성하기 위한 과학기술정보통신부 소프트웨어 분야의 국가 혁신기술 개발형 연구개발 과제이다."
sentences = (
    "성탄 전야 미사를 집전하며 프란치스코 교황이 전한 메시지는 '어린이를 향한 관심'입니다.",
    "프란치스코 교황의 올해 첫 성탄 메시지는 고통받는 어린이를 향한 관심이었습니다.",
)
word_1 = "배"
word_2 = "사과"
NE_text = "노벨 물리학상( - 物理學賞, 스웨덴어: Nobelpriset i fysik, 영어: Nobel Prize in Physics)은 6개 분야의 노벨상 중 하나로, 1년에 한 번 스웨덴 왕립 과학원에 의해 수여된다. 첫 번째 노벨 물리학상은 1901년, 엑스선을 발견한 독일의 빌헬름 콘라트 뢴트겐에게 수여되었다. 이 상은 노벨 재단이 주관하며, 이 상을 수상하는 것은 물리학계에서 최고의 영예로 꼽힌다. 노벨 물리학상은 알프레드 노벨의 사망일인 12월 10일에 스톡홀름에서 수여된다. 2007년의 노벨 물리학상은 프랑스의 알베르 페르와 독일의 페터 그륀베르크가 거대 자기저항의 발견에 대한 공로로 공동 수상하였고, 상금인 1천만 스웨덴 크로나를 나누어 가졌다. 노벨 물리학상은 사람들이 그 과학자의 업적의 중요성을 알기까지의 시간이 걸리기 때문에, 다른 한 편으로는 시간과의 싸움이라고 말할 수 있다. 예를 들어 1983년 노벨 물리학상 수상자인 수브라마니안 찬드라세카르의 경우 그 이론은 이미 1930년에 이미 발표하였지만 사람들에게 인정을 받지 못하여 50여년이 지나서야 상을 받을 수가 있었다. 또한 2013년 노벨 물리학상 수상자인 피터 힉스와 프랑수아 앙글레르의 경우 그의 이론을 검증할 수 있게 되는 과학기술이 발전하기 까지 오랜 시간이 걸려, 49년 뒤 그의 업적이 사실로 확인되어 노벨상을 받게 되었다. 그래서 많은 이론과 발견이 사람들에게 중요성을 인정받았지만, 그 이론이나 발견을 발표한 과학자가 이미 죽어버렸기에 노벨 물리학상을 받지 못하는 안타까운 경우도 있다."
conreference_content = "피시통신은 개인용 컴퓨터(PC)를 다른 컴퓨터와 통신 회선으로 연결하여 자료를 주고 받는 것을 말한다. 개인용 컴퓨터끼리 서로 연결한 통신 형태도 포함되지만, 보통은 정보 서비스 제공을 위한 호스트 컴퓨터와 통신 장비를 설치하고 여기에 가입한 사람들이 개인용 컴퓨터로 접속하여 이용하는 형태의 전화 회선을 통한 통신 네트워크 서비스를 가리킨다. 이때 통신 회선은 주로 전화 모뎀을 통한 전화 회선(PSTN)이 사용되지만 ISDN 등의 다른 회선이 사용되는 경우도 있다. 개인용 컴퓨터가 보편화되면서 1990년대에 게시판과 대화방, 그리고 자료실을 제공하는 PC통신 서비스 회사가 설립되었다."


# 언어 분석
@pytest.mark.asyncio
async def test_analysis(analysis: AnalysisClient):
    r = await analysis.analysis(text=analysis_text, analysis_code=AnalysisCode.morp)
    assert r.data


# 문장 패러프레이즈 인식
@pytest.mark.asyncio
async def test_paraphrase(analysis: AnalysisClient):
    r = await analysis.paraphrase(sentences[0], sentences[1])
    assert r.data


# 어휘 정보
@pytest.mark.asyncio
async def test_wordinfo(analysis: AnalysisClient):
    r = await analysis.wordinfo(word=word_1)
    assert r.data


# 동음이의어 정보
@pytest.mark.asyncio
async def test_homonym(analysis: AnalysisClient):
    r = await analysis.homonym(word=word_1)
    assert r.data


# 다의어 정보
@pytest.mark.asyncio
async def test_polysemy(analysis: AnalysisClient):
    r = await analysis.polysemy(word=word_1)
    assert r.data


# 어휘 간 유사도 분석
@pytest.mark.asyncio
async def test_wordrel(analysis: AnalysisClient):
    r = await analysis.wordrel(first_word=word_1, second_word=word_2)
    assert r.data


# 개체 연결
@pytest.mark.asyncio
async def test_wordrel(analysis: AnalysisClient):
    r = await analysis.nelinking(contents=NE_text)
    assert r[0].data


# 상호참조 해결
@pytest.mark.asyncio
async def test_coreference(analysis: AnalysisClient):
    r = await analysis.coreference(text=conreference_content)
    assert r.data



import pytest
from etripy.client import QAClient, WikiType

text = "가스파초는 어느 나라의 음식인가요?"
mrc_p = "루트비히 판 베토벤(독일어: Ludwig van Beethoven, 문화어: 루드위히 판 베토벤, 1770년 12월 17일 ~ 1827년 3월 26일)은 독일의 서양 고전 음악 작곡가이다. 독일의 본에서 태어났으며, 성인이 된 이후 거의 오스트리아 빈에서 살았다. 감기와 폐렴으로 인한 합병증으로 투병하다가 57세로 생을 마친 그는 고전주의와 낭만주의의 전환기에 활동한 주요 음악가이며, 작곡가로 널리 존경받고 있다. 음악의 성인(聖人) 또는 악성(樂聖)이라는 별칭으로 부르기도 한다. 가장 잘 알려진 작품 가운데에는 〈교향곡 5번〉, 〈교향곡 6번〉, 〈교향곡 9번〉, 〈비창 소나타〉, 〈월광 소나타〉 등이 있다."
mrc_q = "베토벤이 누구야?"
wiki_q = "김구가 누구야?"
legal_q = "대법원장의 임기는 몇년인가?"
doc_key = "2c1b1020-4022-4110-933d-8ef8a77ff64c_8B3E507A7F0001017810168D008A18DA"  # 교통안전법_시행규칙.hwp
doc_q = "교통안전법 시행규칙의 목적은?"


# 질문 분석
@pytest.mark.asyncio
async def test_analysis(qa: QAClient):
    r = await qa.qanal(text=text)
    assert r.data


# 기계 독해
@pytest.mark.asyncio
async def test_mrc(qa: QAClient):
    r = await qa.mrcservlet(question=mrc_q, passage=mrc_p)
    assert r.data


# 위키백과
@pytest.mark.asyncio
async def test_wiki(qa: QAClient):
    r = await qa.wiki(type=WikiType.hybridqa, question=wiki_q)
    assert r.data


# 법률
@pytest.mark.asyncio
async def test_legal(qa: QAClient):
    r = await qa.legal(question=legal_q)
    assert r.data


# 행정문서
@pytest.mark.asyncio
async def test_docfile(qa: QAClient):
    r = await qa.doc(doc_key=doc_key, question=doc_q)
    assert r.data



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from .base import BaseLanguageEtri


@dataclass(frozen=True)
class Morp:
    """형태소 분석 결과"""

    id: Optional[int] = field(repr=True, compare=True, default=None)
    """형태소 ID (출현 순서)"""
    lemma: Optional[str] = field(repr=True, compare=True, default=None)
    """형태소"""
    type: Optional[str] = field(repr=True, compare=True, default=None)
    """형태소 태그"""
    position: Optional[int] = field(repr=True, compare=True, default=None)
    """문장에서의 byte position"""
    weight: Optional[float] = field(repr=True, compare=True, default=None)
    """형태소 분석 결과 신뢰도"""


@dataclass(frozen=True)
class MorpEval:
    id: Optional[int] = field(repr=True, compare=True, default=None)
    result: Optional[str] = field(repr=True, compare=True, default=None)
    target: Optional[str] = field(repr=True, compare=True, default=None)
    word_id: Optional[int] = field(repr=True, compare=True, default=None)
    m_begin: Optional[int] = field(repr=True, compare=True, default=None)
    m_end: Optional[int] = field(repr=True, compare=True, default=None)


@dataclass(frozen=True)
class Wsd:
    """어휘의미 분석 결과"""

    id: Optional[int] = field(repr=True, compare=True, default=None)
    """WSD 대상 ID (출현 순서)"""
    text: Optional[str] = field(repr=True, compare=True, default=None)
    """WSD 대상 어휘 텍스트"""
    type: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘 형태소 태그"""
    scode: Optional[str] = field(repr=True, compare=True, default=None)
    """어휘의 표준국어대사전 어깨번호 (동형이의어 분석 시 2자리, 다의어 분석 시 6자리)"""
    weight: Optional[int] = field(repr=True, compare=True, default=None)
    """어휘의 어휘의미 분석 결과 신뢰도"""
    position: Optional[int] = field(repr=True, compare=True, default=None)
    """문장에서의 byte position"""
    begin: Optional[int] = field(repr=True, compare=True, default=None)
    """어휘의 첫 형태소의 ID"""
    end: Optional[int] = field(repr=True, compare=True, default=None)
    """어휘의 끝 형태소의 ID"""


@dataclass(frozen=True)
class Word:
    """어절 정보 분석 결과"""

    id: Optional[int] = field(repr=True, compare=True, default=None)
    """어절의 ID (출현 순서)"""
    text: Optional[str] = field(repr=True, compare=True, default=None)
    """어절의 대상 텍스트"""
    type: Optional[str] = field(repr=True, compare=True, default=None)
    """어절 타입"""
    begin: Optional[int] = field(repr=True, compare=True, default=None)
    """어절을 구형하는 첫 형태소 ID"""
    end: Optional[int] = field(repr=True, compare=True, default=None)
    """어절을 구형하는 끝 형태소 ID"""


@dataclass(frozen=True)
class Ne:
    """개체명 정보 인식 결과"""

    id: Optional[int] = field(repr=True, compare=True, default=None)
    """개체명 ID (출현 순서)"""
    text: Optional[str] = field(repr=True, compare=True, default=None)
    """개체명 어휘"""
    type: Optional[str] = field(repr=True, compare=True, default=None)
    """개체명 타입"""
    begin: Optional[int] = field(repr=True, compare=True, default=None)
    """개체명을 구성하는 첫 형태소의 ID"""
    end: Optional[int] = field(repr=True, compare=True, default=None)
    """개체명을 구성하는 끝 형태소의 ID"""
    weight: Optional[float] = field(repr=True, compare=True, default=None)
    """개체명 인식 결과 신뢰도"""
    common_noun: Optional[int] = field(repr=True, compare=True, default=None)
    """고유명사인 경우 0 일반명사인 경우 1"""


@dataclass(frozen=True)
class Dependency:
    """의존구문 분석 결과"""

    id: Optional[int] = field(repr=True, compare=True, default=None)
    """어절의 ID (출현 순서)"""
    text: Optional[str] = field(repr=True, compare=True, default=None)
    """의존구문 텍스트"""
    head: Optional[int] = field(repr=True, compare=True, default=None)
    """부모 어절의 ID"""
    label: Optional[str] = field(repr=True, compare=True, default=None)
    """의존관계"""
    mod: Optional[List[int]] = field(repr=True, compare=True, default=None)
    """자식 어절들의 ID"""
    weight: Optional[float] = field(repr=True, compare=True, default=None)
    """의존구문 분석 결과 신뢰도"""


@dataclass(frozen=True)
class PhraseDependency:
    id: Optional[int] = field(repr=True, compare=True, default=None)
    label: Optional[str] = field(repr=True, compare=True, default=None)
    text: Optional[str] = field(repr=True, compare=True, default=None)
    begin: Optional[int] = field(repr=True, compare=True, default=None)
    end: Optional[int] = field(repr=True, compare=True, default=None)
    key_begin: Optional[int] = field(repr=True, compare=True, default=None)
    head_phrase: Optional[int] = field(repr=True, compare=True, default=None)
    sub_phrase: Optional[List[Any]] = field(repr=True, compare=True, default=None)
    weight: Optional[float] = field(repr=True, compare=True, default=None)
    element: Optional[List[Any]] = field(repr=True, compare=True, default=None)


@dataclass(frozen=True)
class SrlArgument:
    """논항 정보"""

    type: Optional[str] = field(repr=True, compare=True, default=None)
    """논항 타입 정보"""
    word_id: Optional[int] = field(repr=True, compare=True, default=None)
    """논항 어절 ID"""
    text: Optional[str] = field(repr=True, compare=True, default=None)
    """논항 대상 어절의 텍스트"""
    weight: Optional[float] = field(repr=True, compare=True, default=None)
    """논항 분석에 대한 신뢰도"""


@dataclass(frozen=True)
class Srl:
    """의미역 분석 결과"""

    verb: Optional[str] = field(repr=True, compare=True, default=None)
    """대상 용언(predicate)"""
    sense: Optional[int] = field(repr=True, compare=True, default=None)
    """용언에 대한 의미 번호"""
    word_id: Optional[int] = field(repr=True, compare=True, default=None)
    """용언 어절 ID"""
    weight: Optional[float] = field(repr=True, compare=True, default=None)
    """결과에 대한 신뢰도"""
    argument: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    """논항 정보"""


    @property
    def Argument(self) -> List[SrlArgument]:
        """논항 정보"""
        if self.argument is None:
            return []
        return [SrlArgument(**argument) for argument in self.argument]


@dataclass(frozen=True)
class Title:
    text: Optional[str] = field(repr=True, compare=True, default=None)
    NE: Optional[str] = field(repr=True, compare=True, default=None)


@dataclass(frozen=True)
class Sentence:
    id: Optional[int] = field(repr=True, compare=True, default=None)
    reserve_str: Optional[str] = field(repr=True, compare=True, default=None)
    text: Optional[str] = field(repr=True, compare=True, default=None)
    morp: Optional[List[Dict[str, Any]]] = field(repr=True, compare=True, default=None)
    morp_eval: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    WSD: Optional[List[Dict[str, Any]]] = field(repr=True, compare=True, default=None)
    word: Optional[List[Dict[str, Any]]] = field(repr=True, compare=True, default=None)
    NE: Optional[List[Dict[str, Any]]] = field(repr=True, compare=True, default=None)
    NE_Link: Optional[List[Any]] = field(repr=True, compare=True, default=None)
    chunk: Optional[List[Any]] = field(repr=True, compare=True, default=None)
    dependency: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    phrase_dependency: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    SRL: Optional[List[Dict[str, Any]]] = field(repr=True, compare=True, default=None)
    relation: Optional[List[Any]] = field(repr=True, compare=True, default=None)
    SA: Optional[List[Any]] = field(repr=True, compare=True, default=None)
    ZA: Optional[List[Any]] = field(repr=True, compare=True, default=None)

    @property
    def Morp(self) -> List[Morp]:
        """형태소 분석 결과"""
        if self.morp is None:
            return []
        return [Morp(**morp) for morp in self.morp]

    @property
    def MorpEval(self) -> List[MorpEval]:
        if self.morp_eval is None:
            return []
        return [MorpEval(**morp_eval) for morp_eval in self.morp_eval]

    @property
    def Wsd(self) -> List[Wsd]:
        """어휘의미 분석 결과"""
        if self.WSD is None:
            return []
        return [Wsd(**wsd) for wsd in self.WSD]

    @property
    def Word(self) -> List[Word]:
        """어절 정보 분석 결과"""
        if self.word is None:
            return []
        return [Word(**word) for word in self.word]

    @property
    def Ne(self) -> List[Ne]:
        """개체명 정보 인식 결과"""
        if self.NE is None:
            return []
        return [Ne(**ne) for ne in self.NE]

    @property
    def Dependency(self) -> List[Dependency]:
        """의존구문 분석 결과"""
        if self.dependency is None:
            return []
        return [Dependency(**dependency) for dependency in self.dependency]

    @property
    def PhraseDependency(self) -> List[PhraseDependency]:
        if self.phrase_dependency is None:
            return []
        return [PhraseDependency(**pb) for pb in self.phrase_dependency]

    @property
    def Srl(self) -> List[Srl]:
        """의미역 분석 결과"""
        if self.SRL is None:
            return []
        return [Srl(**srl) for srl in self.SRL]


@dataclass(frozen=True)
class AnalysisResult(BaseLanguageEtri):
    doc_id: Optional[str] = field(repr=True, compare=True, default=None)
    DCT: Optional[str] = field(repr=True, compare=True, default=None)
    category: Optional[str] = field(repr=True, compare=True, default=None)
    category_weight: Optional[int] = field(repr=True, compare=True, default=None)
    title: Optional[Dict[str, str]] = field(repr=True, compare=True, default=None)
    metaInfo: Optional[Dict[Any, Any]] = field(repr=True, compare=True, default=None)
    paragraphInfo: Optional[List[str]] = field(repr=True, compare=True, default=None)
    sentence: Optional[List[Dict[str, Any]]] = field(
        repr=True, compare=True, default=None
    )
    entity: Optional[List[Any]] = field(repr=True, compare=True, default=None)

    @property
    def Sentence(self) -> List[Sentence]:
        if self.sentence is None:
            return []
        return [Sentence(**sentence) for sentence in self.sentence]

    @property
    def Title(self) -> Title:
        if self.title is None:
            return Title()
        return Title(**self.title)



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional

from .base import BaseQAEtri


####################### 기계 독해 #######################
@dataclass(frozen=True)
class MRCResult(BaseQAEtri):
    """기계독해 결과"""

    @property
    def answer(self) -> Optional[str]:
        """정답"""
        return self.data["answer"]

    @property
    def confidence(self) -> Optional[str]:
        """정답의 신뢰도"""
        return self.data["confidence"]

    @property
    def passage(self) -> Optional[str]:
        """요청한 단락"""
        return self.data["passage"]

    @property
    def question(self) -> Optional[str]:
        """요청한 질문"""
        return self.data["question"]

    @property
    def begin(self) -> Optional[int]:
        """단락 내에서 정답 시작 음절 인덱스"""
        return self.data["begin"]

    @property
    def end(self) -> Optional[int]:
        """단락 내에서 정답의 마지막 음절 인덱스"""
        return self.data["end"]


####################### 위키백과 #######################
@dataclass(frozen=True)
class IRInfo:
    """검색 정보"""

    wiki_title: Optional[str] = field(repr=True, compare=True, default=None)
    """검색 결과의 위키백과 타이틀"""
    sent: Optional[str] = field(repr=True, compare=True, default=None)
    """검색 단락"""
    url: Optional[str] = field(repr=True, compare=True, default=None)
    """위키 url"""


@dataclass(frozen=True)
class WikiAnswerInfo:
    """정답 정보"""

    rank: Optional[int] = field(repr=True, compare=True, default=None)
    """정답 순위"""
    answer: Optional[str] = field(repr=True, compare=True, default=None)
    """정답"""
    confidence: Optional[float] = field(repr=True, compare=True, default=None)
    """정답의 신뢰도"""
    url: Optional[List[str]] = field(repr=True, compare=True, default=None)
    """위키 url"""


@dataclass(frozen=True)
class WiKiResult(BaseQAEtri):
    """검색 정보 결과"""

    @property
    def IRInfo(self) -> Optional[List[IRInfo]]:
        """검색 정보"""
        return [
            IRInfo(**wiki) for wiki in self.data["return_object"]["WikiInfo"]["IRInfo"]
        ]

    @property
    def AnswerInfo(self) -> Optional[List[WikiAnswerInfo]]:
        """정답 정보"""
        return [
            WikiAnswerInfo(**wiki)
            for wiki in self.data["return_object"]["WiKiInfo"]["AnswerInfo"]
        ]


####################### 법률 QA #######################
@dataclass(frozen=True)
class LegalAnswerInfo:
    """응답결과 정보"""

    rank: Optional[int] = field(repr=True, compare=True, default=None)
    """정답/검색 순위"""
    answer: Optional[int] = field(repr=True, compare=True, default=None)
    """정답 (검색결과일 경우 빈 문자열)"""
    source: Optional[str] = field(repr=True, compare=True, default=None)
    """정답의 출처(법률명 및 조제목)"""
    clause: Optional[str] = field(repr=True, compare=True, default=None)
    """조 내용"""
    confidence: Optional[str] = field(repr=True, compare=True, default=None)
    """정답의 스코어"""


@dataclass(frozen=True)
class LegalResult(BaseQAEtri):
    """법률 QA 결과"""

    @property
    def AnswerInfo(self) -> List[LegalAnswerInfo]:
        """응답결과 정보"""
        return [
            LegalAnswerInfo(**answer)
            for answer in self.data["return_object"]["LegalInfo"]["AnswerInfo"]
        ]

    @property
    def RelatedQs(self) -> List[str]:
        """유사질문의 리스트 (비어있을 수 있음)"""
        return [self.data["return_object"]["LegalInfo"]["RelatedQs"]]


####################### 행정문서 QA #######################
@dataclass(frozen=True)
class DocUploadResult(BaseQAEtri):
    """등록된 파일 업로드 결과"""

    @property
    def doc_key(self) -> str:
        """등록된 파일에 대한 고유 document key"""
        return self.data["return_object"]["doc_key"]


@dataclass(frozen=True)
class DocInfo:
    """문서 분석 결과"""

    answer: Optional[str] = field(repr=True, compare=True, default=None)
    """정답"""
    passage: Optional[str] = field(repr=True, compare=True, default=None)
    """정답이 포함된 단락"""
    evidence: Optional[List[str]] = field(repr=True, compare=True, default=None)
    """정답의 근거 문장(리스트)"""
    confidence: Optional[float] = field(repr=True, compare=True, default=None)
    """신뢰도"""
    rank: Optional[int] = field(repr=True, compare=True, default=None)
    """순위"""


@dataclass(frozen=True)
class DocResult(BaseQAEtri):
    """등록된 파일의 분석 결과"""

    @property
    def DocInfo(self) -> List[DocInfo]:
        """문서 분석 결과"""
        return [DocInfo(**doc) for doc in self.data["return_object"]["DocInfo"]]

    @property
    def question(self) -> str:
        """질문"""
        return self.data["return_object"]["question"]



import setuptools
from etripy import __version__

setuptools.setup(
    name="etripy",
    version=__version__,
    license="MIT",
    author="VoidAsMad",
    author_email="star@devksy.xyz",
    description="인공지능 기술을 체험할 수 있는 공공 인공지능 오픈 API Wrapper",
    long_description=open("README.md", "rt", encoding="UTF8").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/VoidAsMad/ETRI",
    packages=setuptools.find_packages(),
    python_requires=">=3.6",
    install_requires=["requests", "aiohttp", "pydub"],
)



import base64
import io
from typing import Optional, Union

from etripy.error import (
    AudioFileException,
    VoicePronunciationException,
    VoiceRecognitionException,
)
from etripy.http import EtriRequest
from etripy.model import LanguageCodeType
from etripy.model.voice import PronunciationResult, RecognitionResult
from pydub import AudioSegment


class VoiceClient(EtriRequest):
    """
    ETRI 음성지능 클라이언트 클래스입니다.
    #### access_key
    [ETRI 포털사이트](https://aiopen.etri.re.kr/)에서 발급받은 `access_key`를 입력합니다.
    """

    def __init__(self, access_key: str) -> None:
        super().__init__(access_key=access_key)

    def __convert_to_raw(self, audioFilePath) -> bytes:
        """오디오 파일을 API가 지원하는 파일의 형식으로 변환합니다."""
        audio = AudioSegment.from_file(audioFilePath)
        audio = audio.set_frame_rate(16000).set_sample_width(2).set_channels(1)

        raw_data = io.BytesIO()
        audio.export(raw_data, format="raw")
        return raw_data.getvalue()

    async def recognition(
        self, language_code: Union[LanguageCodeType, str], audio_path: str
    ) -> Optional[RecognitionResult]:
        """
        ### - 음성인식 기술
        사용자가 발성한 녹음된 입력 음성 데이터(단위 파일 또는 버퍼)를 음성인식 서버로 전달하여 문자(텍스트)로 제공합니다.

        #### Parameter
        `language_code` : 음성인식의 입력 음성 언어 코드입니다. 요청할 수 있는 언어 코드는 LanguageCodeType 클래스를 참조하십시오.\n
        `audio_path` : 음성인식을 할 녹음된 음성파일의 경로입니다.
        """
        try:
            audioContents = base64.b64encode(self.__convert_to_raw(audio_path)).decode(
                "utf8"
            )
        except Exception as e:
            raise AudioFileException(f"오디오 파일을 다시 한번 확인해주세요. : {e}")

        data = {"argument": {"language_code": language_code, "audio": audioContents}}

        result = await self.request(
            method="POST", endpoint="/WiseASR/Recognition", data=data
        )
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise VoiceRecognitionException(result["reason"])
        return RecognitionResult(**result["return_object"])

    async def pronunciation(
        self,
        language_code: Union[LanguageCodeType, str],
        speaker_language_code: Union[LanguageCodeType, str],
        audio_path: str,
        script: Optional[str] = None,
    ) -> Optional[PronunciationResult]:
        """
        ### - 발음평가 기술
        한국인을 비롯한 비원어민 영어 발성 및 외국인의 한국어 음성에 대해 문장별 발음 수준을 측정하여 데이터를 제공합니다. 

        #### Parameter
        `language_code` : 발음평가의 입력 음성 언어 코드입니다. 요청할 수 있는 언어 코드는 korean, english입니다.\n
        `speaker_language_code` : 화자의 음성 언어 코드입니다. 요청할 수 있는 언어 코드는 korean, english입니다.\n
        `audio_path` : 음성인식을 할 녹음된 음성파일의 경로입니다.\n
        `script` : 녹음된 음성파일의 제시 문장입니다. API 요청 시 script가 포함되지 않는 경우 비원어민 영어 음성인식을 수행한 이후 인식 결과에 대한 발음평가 점수를 제공합니다.
        """
        if (
            language_code != LanguageCodeType.korean
            and language_code != LanguageCodeType.english
        ):
            raise Exception("language_type 파라미터는 'korean' 또는 'english'만 사용 가능합니다.")
        if speaker_language_code == LanguageCodeType.korean:
            endpoint = "PronunciationKor"
        elif speaker_language_code == LanguageCodeType.english:
            endpoint - "Pronunciation"
        else:
            raise Exception(
                "author_language_code 파라미터는 'korean' 또는 'english'만 사용 가능합니다."
            )
        try:
            audioContents = base64.b64encode(self.__convert_to_raw(audio_path)).decode(
                "utf8"
            )
        except Exception as e:
            raise AudioFileException(f"오디오 파일을 다시 한번 확인해주세요. : {e}")

        data = {
            "argument": {
                "language_code": language_code,
                "script": script,
                "audio": audioContents,
            }
        }

        result = await self.request(
            method="POST", endpoint=f"/WiseASR/{endpoint}", data=data
        )
        try:
            if result["return_object"] == {}:
                return None
        except KeyError:
            if result["result"] != "0":
                raise VoicePronunciationException(result["reason"])
        return PronunciationResult(**result["return_object"])



from dataclasses import dataclass, field
from typing import Any, Dict, List, Optional


@dataclass(frozen=True)
class BaseLanguageEtri:
    data: Dict[str, Any] = field(repr=False)
    """데이터 Dict"""


@dataclass(frozen=True)
class BaseQAEtri:
    data: Dict[str, Any] = field(repr=False)
    """데이터 Dict"""
